

#ifndef __ES_BUFFER_H__
#define __ES_BUFFER_H__

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#include "es_math.h"
#include "es_type.h"
#include "es_common.h"
#include "es_comm_video.h"
#define ES_INVALID_DPB_SIZE 0x7FFFFFFF

__inline static ES_S32 COMMON_GetPlaneBpp(PIXEL_FORMAT_E pixelFormat, ES_U8 bpp[3]) {
    bpp[0] = bpp[1] = bpp[2] = 0;
    switch (pixelFormat) {
        case PIXEL_FORMAT_A8:
        case PIXEL_FORMAT_R8:
        case PIXEL_FORMAT_YUV400:
            bpp[0] = 8;
            break;
        case PIXEL_FORMAT_I420:
        case PIXEL_FORMAT_YV12:
            bpp[0] = 8;
            bpp[1] = bpp[2] = 2;
            break;
        case PIXEL_FORMAT_NV12:
        case PIXEL_FORMAT_NV21:
            bpp[0] = 8;
            bpp[1] = 4;
            break;
        case PIXEL_FORMAT_A1B5G5R5:
        case PIXEL_FORMAT_A1R5G5B5:
        case PIXEL_FORMAT_A4B4G4R4:
        case PIXEL_FORMAT_A4R4G4B4:
        case PIXEL_FORMAT_B4G4R4A4:
        case PIXEL_FORMAT_B4G4R4X4:
        case PIXEL_FORMAT_B5G5R5A1:
        case PIXEL_FORMAT_B5G5R5X1:
        case PIXEL_FORMAT_B5G6R5:
        case PIXEL_FORMAT_R4G4B4A4:
        case PIXEL_FORMAT_R4G4B4X4:
        case PIXEL_FORMAT_R5G5B5A1:
        case PIXEL_FORMAT_R5G5B5X1:
        case PIXEL_FORMAT_R5G6B5:
        case PIXEL_FORMAT_RG16:
        case PIXEL_FORMAT_X1B5G5R5:
        case PIXEL_FORMAT_X1R5G5B5:
        case PIXEL_FORMAT_X4B4G4R4:
        case PIXEL_FORMAT_X4R4G4B4:
        case PIXEL_FORMAT_YUY2:
        case PIXEL_FORMAT_YVYU:
        case PIXEL_FORMAT_UYVY:
        case PIXEL_FORMAT_VYUY:
        case PIXEL_FORMAT_GRAY16F:
        case PIXEL_FORMAT_YUV400P010BE:
        case PIXEL_FORMAT_YUV400P010LE:
        case PIXEL_FORMAT_YUV400P012BE:
        case PIXEL_FORMAT_YUV400P012LE:
        case PIXEL_FORMAT_YUV400P014BE:
        case PIXEL_FORMAT_YUV400P014LE:
            bpp[0] = 16;
            break;
        case PIXEL_FORMAT_NV16:
        case PIXEL_FORMAT_NV61:
            bpp[0] = bpp[1] = 8;
            break;
        case PIXEL_FORMAT_B8G8R8:
        case PIXEL_FORMAT_B8G8R8I:
        case PIXEL_FORMAT_R8G8B8:
        case PIXEL_FORMAT_R8G8B8I:
            bpp[0] = 24;
            break;
        case PIXEL_FORMAT_B8G8R8_PLANAR:
        case PIXEL_FORMAT_B8G8R8I_PLANAR:
        case PIXEL_FORMAT_R8G8B8_PLANAR:
        case PIXEL_FORMAT_R8G8B8I_PLANAR:
        case PIXEL_FORMAT_YUV444P:
            bpp[0] = bpp[1] = bpp[2] = 8;
            break;
        case PIXEL_FORMAT_YUV420P010BE:
        case PIXEL_FORMAT_YUV420P010LE:
        case PIXEL_FORMAT_YVU420P010BE:
        case PIXEL_FORMAT_YVU420P010LE:
            bpp[0] = 16;
            bpp[1] = bpp[2] = 4;
            break;
        case PIXEL_FORMAT_YUV420SP010BE:
        case PIXEL_FORMAT_YUV420SP010LE:
        case PIXEL_FORMAT_YVU420SP010BE:
        case PIXEL_FORMAT_YVU420SP010LE:
            bpp[0] = 16;
            bpp[1] = 8;
            break;
        case PIXEL_FORMAT_YUV422SP010BE:
        case PIXEL_FORMAT_YUV422SP010LE:
        case PIXEL_FORMAT_YVU422SP010BE:
        case PIXEL_FORMAT_YVU422SP010LE:
            bpp[0] = bpp[1] = 16;
            break;
        case PIXEL_FORMAT_R8G8B8A8:
        case PIXEL_FORMAT_R8G8B8X8:
        case PIXEL_FORMAT_X8B8G8R8:
        case PIXEL_FORMAT_X8R8G8B8:
        case PIXEL_FORMAT_A8B8G8R8:
        case PIXEL_FORMAT_A8R8G8B8:
        case PIXEL_FORMAT_B8G8R8A8:
        case PIXEL_FORMAT_B8G8R8X8:
        case PIXEL_FORMAT_A2B10G10R10:
        case PIXEL_FORMAT_A2R10G10B10:
        case PIXEL_FORMAT_B10G10R10A2:
        case PIXEL_FORMAT_R10G10B10A2:
        case PIXEL_FORMAT_GRAY32F:
        case PIXEL_FORMAT_YUV422I010LE:
            bpp[0] = 32;
            break;
        case PIXEL_FORMAT_B16G16R16:
        case PIXEL_FORMAT_B16G16R16I:
        case PIXEL_FORMAT_R16G16B16:
        case PIXEL_FORMAT_R16G16B16F:
        case PIXEL_FORMAT_R16G16B16I:
            bpp[0] = 48;
            break;
        case PIXEL_FORMAT_B16G16R16_PLANAR:
        case PIXEL_FORMAT_B16G16R16I_PLANAR:
        case PIXEL_FORMAT_R16G16B16_PLANAR:
        case PIXEL_FORMAT_R16G16B16F_PLANAR:
        case PIXEL_FORMAT_R16G16B16I_PLANAR:
            bpp[0] = bpp[1] = bpp[2] = 16;
            break;
        case PIXEL_FORMAT_R32G32B32F:
            bpp[0] = 96;
            break;
        case PIXEL_FORMAT_R32G32B32F_PLANAR:
            bpp[0] = bpp[1] = bpp[2] = 32;
            break;
        default:
            return ES_FAILURE;
    }
    return ES_SUCCESS;
}

__inline static ES_U32 COMMON_GetBpp(PIXEL_FORMAT_E pixelFormat) {
    ES_U8 bpp[3] = {0, 0, 0};

    COMMON_GetPlaneBpp(pixelFormat, bpp);

    return (ES_U32)bpp[0] + bpp[1] + bpp[2];
}

/*******************************************************************************
**
**  COMMON_GetPicBufInfo
**
**  Calculate the stride, offset, plane with given width, height and pixelFormat
**
**  Return image buffer size.
**
**  INPUT:
**      PIXEL_FORMAT_E pixelFormat
**          Image pixel format.
**
**      ES_U32 width
**          Image width.
**
**      ES_U32 height
**          Image height.
**
**      ES_U32 align
**          The requirement minimum align, it also means minimum stride align
**
**      ES_U32 alignHeight
**          Image minimum align height. for example NV12, it should be 2. Some hardware require 8
**
**  OUTPUT:
**
**      ES_U32 *pStride
**          It would return each plane stride if pStride was not ES_NULL
**
**      ES_U32 *pOffset
**          It would return each plane offset if pOffset was not ES_NULL
**
**      ES_U32 *pPlane
**          It return plane value if pPlane was not ES_NULL
**
*/

__inline static ES_U64 COMMON_GetPicBufInfo(PIXEL_FORMAT_E pixelFormat,
                                            ES_U32 width,
                                            ES_U32 height,
                                            ES_U32 align,
                                            ES_U32 alignHeight,
                                            ES_U32 *pStride,
                                            ES_U32 *pOffset,
                                            ES_U32 *pPlane) {
    ES_U32 bpp, plane, stride;
    ES_U32 uStride, vStride, uOffset, vOffset, alignWidth, strideAlign;

    bpp = COMMON_GetBpp(pixelFormat);
    if (!bpp) return 0;
    alignWidth = (align > 0) ? ES_ALIGN_UP(width, align) : width;
    alignHeight = (alignHeight > 0) ? ES_ALIGN_UP(height, alignHeight) : height;
    strideAlign = (align < 2) ? 2 : ES_ALIGN_UP(align, 2);
    stride = ES_ALIGN_UP(alignWidth, strideAlign);

    switch (pixelFormat) {
        case PIXEL_FORMAT_NV12:
        case PIXEL_FORMAT_NV21:
            /*  WxH Y plane followed by (W)x(H/2) interleaved U/V plane. */
            stride = alignWidth;
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = vStride = stride;
            uOffset = vOffset = stride * alignHeight;
            plane = 2;
            break;
        case PIXEL_FORMAT_NV16:
        case PIXEL_FORMAT_NV61:
            stride = alignWidth;
            /*  WxH Y plane followed by WxH interleaved U/V(V/U) plane. */
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = vStride = stride;
            uOffset = vOffset = stride * alignHeight;
            plane = 2;
            break;
        case PIXEL_FORMAT_YUV420SP010LE:
        case PIXEL_FORMAT_YUV420SP010BE:
        case PIXEL_FORMAT_YVU420SP010LE:
        case PIXEL_FORMAT_YVU420SP010BE:
            /*  WxH Y plane followed by (W)x(H/2) interleaved U/V plane. */
            stride = alignWidth * 2;
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = vStride = stride;
            uOffset = vOffset = stride * alignHeight;
            plane = 2;
            break;
        case PIXEL_FORMAT_YUV422SP010LE:
        case PIXEL_FORMAT_YUV422SP010BE:
        case PIXEL_FORMAT_YVU422SP010LE:
        case PIXEL_FORMAT_YVU422SP010BE:
            /*  WxH Y plane followed by WxH interleaved U/V(V/U) plane. */
            stride = alignWidth * 2;
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = vStride = stride;
            uOffset = vOffset = stride * alignHeight;
            plane = 2;
            break;
        case PIXEL_FORMAT_I420:
        case PIXEL_FORMAT_YV12:
            /*  WxH Y plane followed by (W/2)x(H/2) U and V planes. */
            uStride = vStride = (stride / 2);
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = ES_ALIGN_UP(uStride, strideAlign / 2);
            vStride = ES_ALIGN_UP(vStride, strideAlign / 2);
            uOffset = stride * alignHeight;
            vOffset = uOffset + vStride * alignHeight / 2;
            plane = 3;
            break;
        case PIXEL_FORMAT_B8G8R8_PLANAR:
        case PIXEL_FORMAT_B8G8R8I_PLANAR:
        case PIXEL_FORMAT_R8G8B8_PLANAR:
        case PIXEL_FORMAT_R8G8B8I_PLANAR:
        case PIXEL_FORMAT_YUV444P:
            stride = alignWidth;
            uStride = vStride = stride;
            uOffset = stride * alignHeight;
            vOffset = uOffset + uStride * alignHeight;
            plane = 3;
            break;
        case PIXEL_FORMAT_YUV420P010BE:
        case PIXEL_FORMAT_YUV420P010LE:
        case PIXEL_FORMAT_YVU420P010BE:
        case PIXEL_FORMAT_YVU420P010LE:
            /*  WxH Y plane followed by (W/2)x(H/2) U and V planes. */
            stride = alignWidth * 2;
            uStride = vStride = (stride / 2);
            stride = ES_ALIGN_UP(stride, strideAlign);
            uStride = ES_ALIGN_UP(uStride, strideAlign / 2);
            vStride = ES_ALIGN_UP(vStride, strideAlign / 2);
            uOffset = stride * alignHeight;
            vOffset = uOffset + uStride * alignHeight / 2;
            plane = 3;
            break;
        case PIXEL_FORMAT_B16G16R16_PLANAR:
        case PIXEL_FORMAT_B16G16R16I_PLANAR:
        case PIXEL_FORMAT_R16G16B16_PLANAR:
        case PIXEL_FORMAT_R16G16B16F_PLANAR:
        case PIXEL_FORMAT_R16G16B16I_PLANAR:
            stride = alignWidth * 2;
            uStride = vStride = stride;
            uOffset = stride * alignHeight;
            vOffset = uOffset + uStride * alignHeight;
            plane = 3;
            break;
        case PIXEL_FORMAT_R32G32B32F_PLANAR:
            stride = alignWidth * 4;
            uStride = vStride = stride;
            uOffset = stride * alignHeight;
            vOffset = uOffset + uStride * alignHeight;
            plane = 3;
            break;
        default:
            stride = (alignWidth * bpp) / 8;
            uStride = vStride = 0;
            uOffset = vOffset = 0;
            plane = 1;
            break;
    }

    if (pStride) {
        pStride[0] = stride;
        if (plane > 1) pStride[1] = uStride;
        if (plane > 2) pStride[2] = vStride;
    }
    if (pOffset) {
        pOffset[0] = 0;
        if (plane > 1) pOffset[1] = uOffset;
        if (plane > 2) pOffset[2] = vOffset;
    }
    if (pPlane) {
        *pPlane = plane;
    }

    return (ES_U64)alignWidth * alignHeight * bpp / 8;
}

__inline static ES_U64 COMMON_GetPicBufferSize(
    PIXEL_FORMAT_E pixelFormat, ES_U32 width, ES_U32 height, ES_U32 alignWidth, ES_U32 alignHeight) {
    ES_U32 stride, calculateH, pageSize;
    ES_U64 bufSize;

    stride = ES_ALIGN_UP(width, alignWidth);
    calculateH = (alignHeight > 0) ? ES_ALIGN_UP(height, alignHeight) : height;
    bufSize = (ES_U64)stride * calculateH * COMMON_GetBpp(pixelFormat) / 8;
    pageSize = getpagesize();

    return ES_ALIGN_UP(bufSize, pageSize);
}

__inline static ES_U32 COMMON_GetPicOutputSize(
    PIXEL_FORMAT_E pixelFormat, const ES_U32 *pStride, ES_U32 height, const ES_U32 *pOffset, ES_U32 alignHeight) {
    if (!pStride) return 0;
    if (!pOffset) return 0;
    ES_U32 size = 0;
    ES_U32 h = 0;
    if (alignHeight > 0) {
        h = ES_ALIGN_UP(height, alignHeight);
    } else {
        h = height;
    }

    switch (pixelFormat) {
        case PIXEL_FORMAT_GRAY8:
        case PIXEL_FORMAT_YUV400P010LE:
        case PIXEL_FORMAT_YUV400P010BE:
            size = pStride[0] * h + pOffset[0];
            break;
        case PIXEL_FORMAT_YUY2:
        case PIXEL_FORMAT_YVYU:
        case PIXEL_FORMAT_VYUY:
        case PIXEL_FORMAT_UYVY:
        case PIXEL_FORMAT_NV16:
        case PIXEL_FORMAT_NV61:
        case PIXEL_FORMAT_YUV422SP010LE:
        case PIXEL_FORMAT_YUV422SP010BE:
        case PIXEL_FORMAT_YVU422SP010LE:
        case PIXEL_FORMAT_YVU422SP010BE:
            size = pStride[0] * h + pOffset[0];
            size = size < pOffset[1] ? pOffset[1] : size;
            size += pStride[1] * h / 2;
            size = size < pOffset[2] ? pOffset[2] : size;
            size += pStride[2] * h / 2;
            break;
        case PIXEL_FORMAT_YUV420P010LE:
        case PIXEL_FORMAT_YUV420P010BE:
        case PIXEL_FORMAT_YVU420P010LE:
        case PIXEL_FORMAT_YVU420P010BE:
        case PIXEL_FORMAT_I420:
        case PIXEL_FORMAT_YV12:
            size = pStride[0] * h + pOffset[0];
            size = size < pOffset[1] ? pOffset[1] : size;
            size += pStride[1] * h / 2;
            size = size < pOffset[2] ? pOffset[2] : size;
            size += pStride[2] * h / 2;
            break;
        case PIXEL_FORMAT_YUV420SP010LE:
        case PIXEL_FORMAT_YUV420SP010BE:
        case PIXEL_FORMAT_YVU420SP010LE:
        case PIXEL_FORMAT_YVU420SP010BE:
        case PIXEL_FORMAT_NV12:
        case PIXEL_FORMAT_NV21:
            size = pStride[0] * h + pOffset[0];
            size = size < pOffset[1] ? pOffset[1] : size;
            size += pStride[1] * h / 4;
            size = size < pOffset[2] ? pOffset[2] : size;
            size += pStride[2] * h / 4;
            break;
        case PIXEL_FORMAT_B8G8R8:
        case PIXEL_FORMAT_R8G8B8:
        case PIXEL_FORMAT_A8R8G8B8:
        case PIXEL_FORMAT_A8B8G8R8:
        case PIXEL_FORMAT_X8B8G8R8:
        case PIXEL_FORMAT_X8R8G8B8:
        case PIXEL_FORMAT_R16G16B16:
        case PIXEL_FORMAT_B16G16R16:
        case PIXEL_FORMAT_A2R10G10B10:
        case PIXEL_FORMAT_A2B10G10R10:
            size = pStride[0] * h + pOffset[0];
            break;
        default:
            return 0;
    }
    return ES_ALIGN_UP(size, getpagesize());
}

__inline static ES_U32 VI_GetRawBufferSizeEx(
    ES_U32 width, ES_U32 height, PIXEL_FORMAT_E pixelFormat, COMPRESS_MODE_E cmpMode, ES_U32 cmpRatio, ES_U32 align) {
    (ES_VOID) pixelFormat;
    ES_U32 bitWidth = 0;
    ES_U32 size = 0;
    ES_U32 stride = 0;
    ES_U32 rawCmpRatio = 2000;

    if (COMPRESS_MODE_LINE == cmpMode) {
        rawCmpRatio = 2000;

    } else if (COMPRESS_MODE_FRAME == cmpMode) {
        if (0 == cmpRatio) {
            rawCmpRatio = 3000;
        } else {
            rawCmpRatio = cmpRatio;
        }
    }

    /* align: 0 is automatic mode, alignment size following system. Non-0 for specified alignment size */
    if (0 == align) {
        align = ES_DEFAULT_ALIGN;
    } else if (align > ES_MAX_ALIGN) {
        align = ES_MAX_ALIGN;
    } else {
        align = (ES_ALIGN_UP(align, ES_DEFAULT_ALIGN));
    }

#if 0
    switch (pixelFormat) {
        case PIXEL_FORMAT_RGB_BAYER_8BPP: {
            bitWidth = 8;
            break;
        }
        default: {
            bitWidth = 0;
            break;
        }
    }
#endif

    if (COMPRESS_MODE_NONE == cmpMode) {
        stride = ES_ALIGN_UP(ES_ALIGN_UP(width * bitWidth, 8) / 8, align);
        size = stride * height;
    } else if (COMPRESS_MODE_LINE == cmpMode) {
        ES_U32 tmp;
        tmp = ES_ALIGN_UP((16 + width * bitWidth * 1000ULL / rawCmpRatio + 8192 + 127) / 128, 2);
        stride = ES_ALIGN_UP(tmp * 16, align);
        size = stride * height;
    } else if (COMPRESS_MODE_FRAME == cmpMode) {
        size = ES_ALIGN_UP(height * width * bitWidth * 1000ULL / (rawCmpRatio * 8), align);
    }

    return size;
}

__inline static ES_U32 VI_GetRawBufferSize(
    ES_U32 width, ES_U32 height, PIXEL_FORMAT_E pixelFormat, COMPRESS_MODE_E cmpMode, ES_U32 align) {
    ES_U32 size = 0;

    size = VI_GetRawBufferSizeEx(width, height, pixelFormat, cmpMode, 0, align);

    return size;
}

__inline static ES_U32 AVS_GetPicBufferSize(ES_U32 width, ES_U32 height, COMPRESS_MODE_E cmpMode, ES_U32 align) {
    (ES_VOID) cmpMode;
    (ES_VOID) align;
    ES_U32 size = 0;

    size = ES_ALIGN_UP(width, 128) * ES_ALIGN_UP(height, 64) * 2;

    return size;
}

/*
 * Parameter:
 * @picWidthInMbs: SPS->seq_paramter_set_data->pic_width_in_mbs_minus1 + 1
 * @picHeightInMbs: SPS->seq_paramter_set_data->pic_height_in_maps_units_minus1 + 1
 * @levelIdc: SPS->seq_paramter_set_data->level_idc
 * @pNumRefFrames: SPS->seq_paramter_set_data->max_num_ref_frames, it is array with length 2 when mvcStream == ES_TRUE
 * @noReorder: whether decoder order output
 * @skipNonIntra: whether skip non intra frame
 * @mvcStream: whether mvc stream
 * @pPicOrderCntType: SPS->seq_paramter_set_data->pic_order_cnt_type, , it is array with length 2 when mvcStream ==
 * ES_TRUE
 * @pVUIParamPresentFlag: SPS->seq_paramter_set_data->vui_paramters_present_flag, it is array with length 2 when
 * mvcStream == ES_TRUE
 * @pBitStreamRestrictionFlag: SPS->seq_paramter_set_data->vui_paramters_present_flag->bitstream_restriction_flag, it is
 * array with length 2 when mvcStream == ES_TRUE
 * @pNumReorderFrames:
 * SPS->seq_paramter_set_data->vui_paramters_present_flag->bitstream_restriction_flag->num_reorder_frames, it is array
 * with length 2 when mvcStream == ES_TRUE
 * @nCores: decoder core count
 * @mutilSPSParam: wether multi sps with mvcStream == ES_TRUE
 */
__inline static ES_U32 VDEC_GetH264RefBufferCount(ES_U32 picWidthInMbs,
                                                  ES_U32 picHeightInMbs,
                                                  ES_U32 levelIdc,
                                                  const ES_U32 *pNumRefFrames,
                                                  ES_BOOL noReorder,
                                                  ES_BOOL skipNonIntra,
                                                  ES_BOOL mvcStream,
                                                  const ES_U32 *pPicOrderCntType,
                                                  const ES_U32 *pVUIParamPresentFlag,
                                                  const ES_U32 *pBitStreamRestrictionFlag,
                                                  const ES_U32 *pNumReorderFrames,
                                                  ES_U32 nCores,
                                                  ES_BOOL mutilSPSParam) {
    ES_U32 picSizeInMbs = picWidthInMbs * picHeightInMbs;
    /* Variables */
    ES_U32 tmp = 0;
    ES_U32 maxPicSizeInMbs = 0;
    ES_U32 DPBSize = 0;
    ES_U32 maxDPBSize = 0;
    ES_U32 minBufferNum = 0;

    if (picSizeInMbs == 0) {
        DPBSize = ES_INVALID_DPB_SIZE;
    } else {
        /* use tmp as the size of the DPB in bytes, computes as 1024 * MaxDPB
         * (from table A-1 in Annex A) */
        switch (levelIdc) {
            case 10:
                tmp = 396;
                maxPicSizeInMbs = 99;
                break;

            case 11:
                tmp = 900;
                maxPicSizeInMbs = 396;
                break;

            case 12:
                tmp = 2376;
                maxPicSizeInMbs = 396;
                break;

            case 13:
                tmp = 2376;
                maxPicSizeInMbs = 396;
                break;

            case 20:
                tmp = 2376;
                maxPicSizeInMbs = 396;
                break;

            case 21:
                tmp = 4752;
                maxPicSizeInMbs = 792;
                break;

            case 22:
                tmp = 8100;
                maxPicSizeInMbs = 1620;
                break;

            case 30:
                tmp = 8100;
                maxPicSizeInMbs = 1620;
                break;

            case 31:
                tmp = 18000;
                maxPicSizeInMbs = 3600;
                break;

            case 32:
                tmp = 20480;
                maxPicSizeInMbs = 5120;
                break;

            case 40:
                tmp = 32768;
                maxPicSizeInMbs = 8192;
                break;

            case 41:
                tmp = 32768;
                maxPicSizeInMbs = 8192;
                break;

            case 42:
                tmp = 34816;
                maxPicSizeInMbs = 8704;
                break;

            case 50:
                tmp = 110400;
                maxPicSizeInMbs = 22080;
                break;

            case 51:
                tmp = 184320;
                maxPicSizeInMbs = 36864;
                break;

            case 52:
                tmp = 184320;
                maxPicSizeInMbs = 36864;
                break;

            case 60:
                tmp = 696320;
                maxPicSizeInMbs = 139264;
                break;

            case 61:
                tmp = 696320;
                maxPicSizeInMbs = 139264;
                break;

            case 62:
                tmp = 696320;
                maxPicSizeInMbs = 139264;
                break;

            default:
                DPBSize = ES_INVALID_DPB_SIZE;
        }

        if (picSizeInMbs > maxPicSizeInMbs) {
            DPBSize = ES_INVALID_DPB_SIZE;
        }
        if (DPBSize == 0) {
            tmp /= picSizeInMbs;
            DPBSize = ES_MIN2(tmp, 16);
        }
    }

    if (DPBSize == ES_INVALID_DPB_SIZE || pNumRefFrames[0] > DPBSize) {
        /* set max_dpb_size to 1 if num_ref_frames is zero */
        DPBSize = pNumRefFrames[0] ? pNumRefFrames[0] : 1;
    }
    maxDPBSize = DPBSize;

    if (!mvcStream) {
        ES_U32 totalBuffers = 0;
        if (noReorder)
            totalBuffers = ES_MAX2(pNumRefFrames[0], 1) + 1;
        else
            totalBuffers = maxDPBSize + 1;

        if (skipNonIntra) {
            totalBuffers = 2;
        }

        if (nCores > 1) {
            totalBuffers += nCores;
        }
        if (totalBuffers > 34) totalBuffers = 34;
        minBufferNum = totalBuffers;
    } else {
        ES_U32 j = 0;
        for (ES_U32 i = 0; i < 2; i++) {
            ES_U32 currentNoReorder, totalBuffers;
            if (noReorder || pPicOrderCntType[j] == 2
                || (pVUIParamPresentFlag[j] && pBitStreamRestrictionFlag[j] && !pNumReorderFrames[j]))
                currentNoReorder = ES_TRUE;
            else
                currentNoReorder = ES_FALSE;

            /* restrict max dpb size of mvc (stereo high) streams, make sure that
             * base address 15 is available/restricted for inter view reference use */
            maxDPBSize = ES_MIN2(maxDPBSize, 8);
            if (currentNoReorder) {
                totalBuffers = ES_MAX2(pNumRefFrames[j], 1) + 1;
            } else {
                totalBuffers = maxDPBSize + 1;
            }

            minBufferNum += totalBuffers;
            if (mutilSPSParam) j++;
        }
    }
    return minBufferNum;
}

/*
 * Parameter:
 * @maxPicBuffering:SPS->seq_paramter_set_rbsp->SubLayers->sps_max_dec_pic_buffering_minus1 + 1
 * @skipNonIntra: whether skip non intra frame
 */
__inline static ES_U32 VDEC_GetH265RefBufferCount(ES_U32 maxPicBuffering, ES_BOOL skipNonIntra) {
    return skipNonIntra ? 2 : ES_MAX2(1, maxPicBuffering) + 1;
}

__inline static ES_U64 VDEC_GetPicBufferSize(PIXEL_FORMAT_E pixelFormat, ES_U32 width, ES_U32 height, ES_U32 align) {
    return COMMON_GetPicBufferSize(pixelFormat, width, height, align, 0);
}

__inline static ES_S32 COMMON_GetDataBitWidth(PIXEL_FORMAT_E pixelFmt) {
    ES_S32 bitWidth = 0;
    switch (pixelFmt) {
        case PIXEL_FORMAT_GRAY8:
        case PIXEL_FORMAT_I420:
        case PIXEL_FORMAT_YV12:
        case PIXEL_FORMAT_NV12:
        case PIXEL_FORMAT_NV21:
        case PIXEL_FORMAT_YUY2:
        case PIXEL_FORMAT_YVYU:
        case PIXEL_FORMAT_VYUY:
        case PIXEL_FORMAT_UYVY:
        case PIXEL_FORMAT_B8G8R8:
        case PIXEL_FORMAT_R8G8B8:
        case PIXEL_FORMAT_A8R8G8B8:
        case PIXEL_FORMAT_A8B8G8R8:
        case PIXEL_FORMAT_X8B8G8R8:
        case PIXEL_FORMAT_X8R8G8B8:
            bitWidth = 8;
            break;
        case PIXEL_FORMAT_YUV400P010LE:
        case PIXEL_FORMAT_YUV400P010BE:
        case PIXEL_FORMAT_YUV420P010LE:
        case PIXEL_FORMAT_YUV420P010BE:
        case PIXEL_FORMAT_YVU420P010LE:
        case PIXEL_FORMAT_YVU420P010BE:
        case PIXEL_FORMAT_YUV420SP010LE:
        case PIXEL_FORMAT_YUV420SP010BE:
        case PIXEL_FORMAT_YVU420SP010LE:
        case PIXEL_FORMAT_YVU420SP010BE:
        case PIXEL_FORMAT_A2R10G10B10:
        case PIXEL_FORMAT_A2B10G10R10:
            bitWidth = 10;
            break;
        case PIXEL_FORMAT_R16G16B16:
        case PIXEL_FORMAT_B16G16R16:
            bitWidth = 16;
            break;
        default:
            break;
    }
    return bitWidth;
}

__inline static ES_S32 VENC_GetRefBufferCount(PAYLOAD_TYPE_E type, ES_U32 coreNum, ES_U32 refFrameAmount) {
    ES_S32 count = 0;

    if (PT_H264 == type || PT_H265 == type) {
        count = refFrameAmount + coreNum;
    }
    return count;
}

__inline static ES_U32 VENC_GetRefBufferSize(
    PAYLOAD_TYPE_E type, ES_U32 width, ES_U32 height, PIXEL_FORMAT_E pixelFmt, ES_U32 align) {
    ES_S32 size = 0;

    if (PT_H264 == type || PT_H265 == type) {
        ES_S32 w = width;
        ES_S32 h = height;
        ES_S32 bitWidth = COMMON_GetDataBitWidth(pixelFmt);
        ES_S32 w_4n, h_4n;
        ES_U32 lumaSize, lumaSize4N, frameStride;
        ES_U32 pageSize = getpagesize();
        ES_U32 aligment = align ? align : 64;

        w = ((w + 63) >> 6) << 6;
        h = ((h + 63) >> 6) << 6;
        w_4n = ((width + 15) / 16) * 4;
        h_4n = h / 4;

        frameStride = ES_ALIGN_UP(ES_ALIGN_UP(w * 4 * bitWidth / 8, 16), aligment);
        lumaSize = frameStride * h / 4;
        lumaSize4N = ES_ALIGN_UP(ES_ALIGN_UP(w_4n * 4 * bitWidth / 8, 16), aligment) * h_4n / 4;
        size = ((lumaSize + lumaSize4N + 0xf) & (~0xf));
        size = ES_ALIGN_UP(size, aligment);
        size = ES_ALIGN_UP(size, pageSize);
    }
    return size;
}

__inline static ES_U32 VENC_GetRefPicInfoBufferSize(
    PAYLOAD_TYPE_E type, ES_U32 width, ES_U32 height, PIXEL_FORMAT_E pixelFmt, ES_U32 align) {
    ES_S32 size = 0;
    if (PT_H264 == type || PT_H265 == type) {
        size = VENC_GetRefBufferSize(type, width, height, pixelFmt, align) / 2;
        return size;
    }

    return size;
}

__inline static ES_U32 VENC_GetQpmapSizeStride(ES_U32 width) {
    return ES_DIV_UP(width, 512) * 32;
}

__inline static ES_U32 VENC_GetQpmapSize(ES_U32 width, ES_U32 height) {
    ES_U32 stride, alignHeight;

    stride = VENC_GetQpmapSizeStride(width);
    alignHeight = ES_DIV_UP(height, 16);

    return stride * alignHeight;
}

__inline static ES_U32 VENC_GetSkipWeightSizeStride(PAYLOAD_TYPE_E type, ES_U32 width) {
    ES_U32 stride;

    if (PT_H265 == type) {
        stride = ES_DIV_UP(width, 2048) * 16;
    } else if (PT_H264 == type) {
        stride = ES_DIV_UP(width, 512) * 16;
    } else {
        stride = 0;
    }

    return stride;
}
__inline static ES_U32 VENC_GetSkipWeightSize(PAYLOAD_TYPE_E type, ES_U32 width, ES_U32 height) {
    ES_U32 stride, alignHeight;

    stride = VENC_GetSkipWeightSizeStride(type, width);

    if (PT_H265 == type) {
        alignHeight = ES_DIV_UP(height, 64);
    } else if (PT_H264 == type) {
        alignHeight = ES_DIV_UP(height, 16);
    } else {
        alignHeight = 0;
    }

    return stride * alignHeight;
}

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __ES_BUFFER_H__ */
