
#ifndef __ES_COMM_VIDEO_H__
#define __ES_COMM_VIDEO_H__

#include "es_type.h"
#include "es_common.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#define SRC_LENS_COEF_SEG 2
#define DST_LENS_COEF_SEG 3
#define SRC_LENS_COEF_NUM 4
#define DST_LENS_COEF_NUM 4

#define ISP_BAYER_CHN (4)

typedef enum esOPERATION_MODE_E {
    OPERATION_MODE_AUTO = 0,
    OPERATION_MODE_MANUAL = 1,
    OPERATION_MODE_BUTT
} OPERATION_MODE_E;

typedef enum esROTATION_E {
    ROTATION_0 = 0,
    ROTATION_90 = 1,
    ROTATION_180 = 2,
    ROTATION_270 = 3,
    ROTATION_FLIP_X = 4,
    ROTATION_FLIP_Y = 5,
    ROTATION_BUTT
} ROTATION_E;

typedef enum esVB_SOURCE_E {
    VB_SOURCE_COMMON = 0,
    VB_SOURCE_MODULE = 1,
    VB_SOURCE_PRIVATE = 2,
    VB_SOURCE_USER = 3,
    VB_SOURCE_BUTT
} VB_SOURCE_E;

typedef enum esDATA_RATE_E {
    DATA_RATE_X1 = 0, /* RW; output 1 pixel per clock */
    DATA_RATE_X2 = 1, /* RW; output 2 pixel per clock */
    DATA_RATE_BUTT
} DATA_RATE_E;

typedef struct esBORDER_S {
    ES_U32 topWidth;
    ES_U32 bottomWidth;
    ES_U32 leftWidth;
    ES_U32 rightWidth;
    ES_U32 color;
} BORDER_S;

typedef struct esPOINT_S {
    ES_S32 x;
    ES_S32 y;
} POINT_S;

typedef struct esSIZE_S {
    ES_U32 width;
    ES_U32 height;
} SIZE_S;

typedef struct esRECT_S {
    ES_S32 x;
    ES_S32 y;
    ES_U32 width;
    ES_U32 height;
} RECT_S;

typedef struct esVIDEO_REGION_INFO_S {
    ES_U32 regionNum;          /* W; count of the region */
    RECT_S ATTRIBUTE* pRegion; /* W; region attribute */
} VIDEO_REGION_INFO_S;

typedef struct esCROP_INFO_S {
    ES_BOOL bEnable;
    RECT_S rect;
} CROP_INFO_S;

typedef struct esFRAME_RATE_CTRL_S {
    ES_S32 srcFrameRate; /* RW; source frame rate */
    ES_S32 dstFrameRate; /* RW; dest frame rate */
} FRAME_RATE_CTRL_S;

typedef enum esASPECT_RATIO_E {
    ASPECT_RATIO_NONE = 0,   /* full screen */
    ASPECT_RATIO_AUTO = 1,   /* ratio no change, 1:1*/
    ASPECT_RATIO_MANUAL = 2, /* ratio manual set */
    ASPECT_RATIO_BUTT
} ASPECT_RATIO_E;

typedef struct esASPECT_RATIO_S {
    ASPECT_RATIO_E mode; /* aspect ratio mode: none/auto/manual */
    ES_U32 bgColor;      /* background color, RGB 888 */
    RECT_S videoRect;    /* valid in ASPECT_RATIO_MANUAL mode */
} ASPECT_RATIO_S;

typedef enum esPIXEL_FORMAT_E {
    /* Unknown format. */
    PIXEL_FORMAT_UNKNOWN = 0,

    /* Palettized formats. */
    PIXEL_FORMAT_INDEX8 = 1,

    /* RGB formats. */
    PIXEL_FORMAT_X4R4G4B4 = 100,
    PIXEL_FORMAT_A4R4G4B4,
    PIXEL_FORMAT_X1R5G5B5,
    PIXEL_FORMAT_A1R5G5B5,
    PIXEL_FORMAT_R5G6B5,
    PIXEL_FORMAT_R8G8B8,
    PIXEL_FORMAT_R8G8B8_PLANAR,
    PIXEL_FORMAT_R16G16B16,
    PIXEL_FORMAT_R16G16B16_PLANAR,
    PIXEL_FORMAT_X8R8G8B8,
    PIXEL_FORMAT_A8R8G8B8,
    PIXEL_FORMAT_A2R10G10B10,
    PIXEL_FORMAT_R8G8B8I,
    PIXEL_FORMAT_R8G8B8I_PLANAR,
    PIXEL_FORMAT_R16G16B16I,
    PIXEL_FORMAT_R16G16B16I_PLANAR,
    PIXEL_FORMAT_R4G4B4X4,
    PIXEL_FORMAT_R4G4B4A4,
    PIXEL_FORMAT_R5G5B5X1,
    PIXEL_FORMAT_R5G5B5A1,
    PIXEL_FORMAT_R8G8B8X8,
    PIXEL_FORMAT_R8G8B8A8,
    PIXEL_FORMAT_R10G10B10A2,

    /* BGR formats. */
    PIXEL_FORMAT_X4B4G4R4 = 200,
    PIXEL_FORMAT_A4B4G4R4,
    PIXEL_FORMAT_X1B5G5R5,
    PIXEL_FORMAT_A1B5G5R5,
    PIXEL_FORMAT_B5G6R5,
    PIXEL_FORMAT_B8G8R8,
    PIXEL_FORMAT_B8G8R8_PLANAR,
    PIXEL_FORMAT_B16G16R16,
    PIXEL_FORMAT_B16G16R16_PLANAR,
    PIXEL_FORMAT_X8B8G8R8,
    PIXEL_FORMAT_A8B8G8R8,
    PIXEL_FORMAT_A2B10G10R10,
    PIXEL_FORMAT_B8G8R8I,
    PIXEL_FORMAT_B8G8R8I_PLANAR,
    PIXEL_FORMAT_B16G16R16I,
    PIXEL_FORMAT_B16G16R16I_PLANAR,
    PIXEL_FORMAT_B4G4R4X4,
    PIXEL_FORMAT_B4G4R4A4,
    PIXEL_FORMAT_B5G5R5X1,
    PIXEL_FORMAT_B5G5R5A1,
    PIXEL_FORMAT_B8G8R8X8,
    PIXEL_FORMAT_B8G8R8A8,
    PIXEL_FORMAT_B10G10R10A2,

    /* YUV formats. */
    PIXEL_FORMAT_YUY2 = 300, /* packed YUV 4:2:2, 16bpp, Y0 U0 Y1 V0 */
    PIXEL_FORMAT_UYVY,       /* packed YUV 4:2:2, 16bpp, U0 Y0 V0 Y1 */
    PIXEL_FORMAT_YVYU,       /* packed YUV 4:2:2, 16bpp, Y0 V0 Y1 U0 */
    PIXEL_FORMAT_VYUY,       /* packed YUV 4:2:2, 16bpp, V0 Y0 U0 Y1 */
    PIXEL_FORMAT_NV16,       /* planar YUV 4:2:2, 16bpp, 1st plane for Y and 2nd plane for the UV */
    PIXEL_FORMAT_NV61,       /* planar YUV 4:2:2, 16bpp, 1st plane for Y and 2nd plane for the VU */
    PIXEL_FORMAT_YV12,       /* planar YUV 4:2:0, 12bpp, 1st plane for Y, 2nd plane for V, 3rd plane for U */
    PIXEL_FORMAT_I420,       /* planar YUV 4:2:0, 12bpp, 1st plane for Y, 2nd plane for U, 3rd plane for V */
    PIXEL_FORMAT_NV12,       /* planar YUV 4:2:0, 12bpp, 1st plane for Y and 2nd plane for the UV */
    PIXEL_FORMAT_NV21,       /* planar YUV 4:2:0, 12bpp, 1st plane for Y and 2nd plane for the VU */

    PIXEL_FORMAT_YUV420P010LE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for U, 3rd plane for V, 10bit
                                  stored in 2bytes as little-endian */
    PIXEL_FORMAT_YUV420P010BE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for U, 3rd plane for V, 10bit
                                  stored in 2bytes as big-endian */
    PIXEL_FORMAT_YVU420P010LE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for V, 3rd plane for U, 10bit
                                  stored in 2bytes as little-endian */
    PIXEL_FORMAT_YVU420P010BE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for V, 3rd plane for U, 10bit
                                  stored in 2bytes as big-endian */

    PIXEL_FORMAT_YUV420SP010LE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for UV, 10bit stored in 2bytes
                                   as little-endian */
    PIXEL_FORMAT_YUV420SP010BE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for UV, 10bit stored in 2bytes
                                   as big-endian */
    PIXEL_FORMAT_YVU420SP010LE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for VU, 10bit stored in 2bytes
                                   as little-endian */
    PIXEL_FORMAT_YVU420SP010BE, /* planar YUV 4:2:0, 24bpp, 1st plane for Y, 2nd plane for VU, 10bit stored in 2bytes
                                   as big-endian */

    PIXEL_FORMAT_YUV422SP010LE, /* planar YUV 4:2:2, 32bpp, 1st plane for Y, 2nd plane for UV, 10bit stored in 2bytes
                                   as little-endian */
    PIXEL_FORMAT_YUV422SP010BE, /* planar YUV 4:2:2, 32bpp, 1st plane for Y, 2nd plane for UV, 10bit stored in 2bytes
                                   as big-endian */
    PIXEL_FORMAT_YVU422SP010LE, /* planar YUV 4:2:2, 32bpp, 1st plane for Y, 2nd plane for VU, 10bit stored in 2bytes
                                   as little-endian */
    PIXEL_FORMAT_YVU422SP010BE, /* planar YUV 4:2:2, 32bpp, 1st plane for Y, 2nd plane for VU, 10bit stored in 2bytes
                                   as big-endian */
    PIXEL_FORMAT_YUV422I010LE,  /* packed YUV 4:2:2, 32bpp, Y0 U0 Y1 V0, 10bit stored in 2bytes as little-endian */

    PIXEL_FORMAT_YUV400,       /* planar YUV 4:0:0, 8bpp, only 1 plane for Y */
    PIXEL_FORMAT_YUV400P010LE, /* planar YUV 4:0:0, 10bpp, only 1 plane for Y, 10bit stored in 2bytes as little-endian
                                */
    PIXEL_FORMAT_YUV400P010BE, /* planar YUV 4:0:0, 10bpp, only 1 plane for Y, 10bit stored in 2bytes as big-endian */
    PIXEL_FORMAT_YUV400P012LE, /* planar YUV 4:0:0, 12bpp, only 1 plane for Y, 12bit stored in 2bytes as little-endian
                                */
    PIXEL_FORMAT_YUV400P012BE, /* planar YUV 4:0:0, 12bpp, only 1 plane for Y, 12bit stored in 2bytes as big-endian */
    PIXEL_FORMAT_YUV400P014LE, /* planar YUV 4:0:0, 14bpp, only 1 plane for Y, 14bit stored in 2bytes as little-endian
                                */
    PIXEL_FORMAT_YUV400P014BE, /* planar YUV 4:0:0, 14bpp, only 1 plane for Y, 14bit stored in 2bytes as big-endian */

    PIXEL_FORMAT_YUV444P, /* planar YUV 4:4:4, 24bpp, 1st plane for Y, 2nd plane for U, 3rd plane for V */

    PIXEL_FORMAT_GRAY8 = PIXEL_FORMAT_YUV400,
    PIXEL_FORMAT_GRAY10 = PIXEL_FORMAT_YUV400P012LE,
    PIXEL_FORMAT_GRAY12 = PIXEL_FORMAT_YUV400P014LE,
    PIXEL_FORMAT_GRAY16,

    /* Alpha formats. */
    PIXEL_FORMAT_A8 = 400,

    /* R/RG/RA formats. */
    PIXEL_FORMAT_R8 = 500,
    PIXEL_FORMAT_RG16,

    /* Floating point formats. */
    PIXEL_FORMAT_R16G16B16F = 600,
    PIXEL_FORMAT_R32G32B32F,
    PIXEL_FORMAT_GRAY16F,
    PIXEL_FORMAT_GRAY32F,
    PIXEL_FORMAT_R16G16B16F_PLANAR,
    PIXEL_FORMAT_R32G32B32F_PLANAR,
    PIXEL_FORMAT_BUTT
} PIXEL_FORMAT_E;

typedef enum esVIDEO_FIELD_E {
    VIDEO_FIELD_TOP = 0x1,        /* even field */
    VIDEO_FIELD_BOTTOM = 0x2,     /* odd field */
    VIDEO_FIELD_INTERLACED = 0x3, /* two interlaced fields */
    VIDEO_FIELD_FRAME = 0x4,      /* frame */
    VIDEO_FIELD_BUTT
} VIDEO_FIELD_E;

typedef enum esVIDEO_FORMAT_E {
    VIDEO_FORMAT_LINEAR = 0,     /* nature video line */
    VIDEO_FORMAT_TILE_64x16 = 1, /* tile cell: 64pixel x 16line */
    VIDEO_FORMAT_BUTT
} VIDEO_FORMAT_E;

typedef enum esCOMPRESS_MODE_E {
    COMPRESS_MODE_NONE = 0,  /* no compress */
    COMPRESS_MODE_TILE = 1,  /* compress unit is a tile.*/
    COMPRESS_MODE_LINE = 2,  /* compress unit is the whole line.  raw for VI  */
    COMPRESS_MODE_FRAME = 3, /* compress unit is the whole frame. YUV for VI(3DNR), RGB for TDE(write)/VO(read) */
    COMPRESS_MODE_BUTT
} COMPRESS_MODE_E;

typedef enum esVIDEO_DISPLAY_MODE_E {
    VIDEO_DISPLAY_MODE_PREVIEW = 0x0,
    VIDEO_DISPLAY_MODE_PLAYBACK = 0x1,
    VIDEO_DISPLAY_MODE_BUTT
} VIDEO_DISPLAY_MODE_E;

typedef struct esLUMA_INFO_S {
    ES_U64 lumaPixSum;     /* Luma sum of current frame */
    ES_U32 lumaPixAverage; /* Luma average of current frame */
    ES_U64 PTS;            /* PTS of current frame  */
} LUMA_INFO_S;

#define DCF_DRSCRIPTION_LENGTH 32
#define DCF_CAPTURE_TIME_LENGTH 20

typedef struct esISP_DCF_CONST_INFO_S {
    ES_U8 imageDescription[DCF_DRSCRIPTION_LENGTH]; /*Describes image*/
    ES_U8 make[DCF_DRSCRIPTION_LENGTH];             /*Shows manufacturer of digital cameras*/
    ES_U8 model[DCF_DRSCRIPTION_LENGTH];            /*Shows model number of digital cameras*/
    ES_U8 software[DCF_DRSCRIPTION_LENGTH]; /*Shows firmware (internal software of digital cameras) version number*/

    ES_U8 lightSource;    /*Light source, actually this means white balance setting. '0' means unknown, '1' daylight,
                                '2'   fluorescent, '3' tungsten, '10' flash, '17' standard light A, '18' standard light B,
                                '19' standard light   C, '20' D55, '21' D65, '22' D75, '255' other*/
    ES_U32 focalLength;   /*Focal length of lens used to take image. Unit is millimeter*/
    ES_U8 sceneType;      /*Indicates the type of scene. Value '0x01' means that the image was directly photographed.*/
    ES_U8 customRendered; /*Indicates the use of special processing on image data, such as rendering geared to
                                output.        0 = Normal process  1 = Custom process   */
    ES_U8 focalLengthIn35mmFilm; /*Indicates the equivalent focal length assuming a 35mm film camera, in mm*/
    ES_U8 sceneCaptureType; /*Indicates the type of scene that was shot. 0 = Standard,1 = Landscape,2 = Portrait,3 =
                                  Night scene. */
    ES_U8 gainControl;      /*Indicates the degree of overall image gain adjustment. 0 = None,1 = Low gain up,2 = High
                                  gain up,3 = Low gain down,4 = High gain down. */
    ES_U8 contrast;     /*Indicates the direction of contrast processing applied by the camera when the image was shot.
                                     0 = Normal,1 = Soft,2 = Hard */
    ES_U8 saturation;   /*Indicates the direction of saturation processing applied by the camera when the image was
                              shot. 0 = Normal,1 = Low saturation,2 = High saturation*/
    ES_U8 sharpness;    /*Indicates the direction of sharpness processing applied by the camera when the image was shot.
                                    0 = Normal,1 = Soft,2 = Hard .*/
    ES_U8 meteringMode; /*Exposure metering method. '0' means unknown, '1' average, '2' center weighted average, '3'
                                    spot, '4' multi-spot, '5' multi-segment, '6' partial, '255' other*/
} ISP_DCF_CONST_INFO_S;

typedef struct esISP_DCF_UPDATE_INFO_S {
    ES_U32 ISOSpeedRatings;   /*CCD sensitivity equivalent to Ag-Hr film speedrate*/
    ES_U32 exposureTime;      /*Exposure time (reciprocal of shutter speed).*/
    ES_U32 exposureBiasValue; /*Exposure bias (compensation) value of taking picture*/
    ES_U8 exposureProgram;    /*Exposure program that the camera used when image was taken. '1' means manual control,
                                    '2'    program normal, '3' aperture priority, '4' shutter priority, '5' program
                                    creative    (slow program),    '6' program action(high-speed program), '7' portrait
                                    mode,    '8'    landscape mode*/
    ES_U32 fNumber;           /*The actual F-number (F-stop) of lens when the image was taken*/
    ES_U32 maxApertureValue;  /*Maximum aperture value of lens.*/
    ES_U8 exposureMode;       /*Indicates the exposure mode set when the image was shot.
                                          0 = Auto exposure,1 = Manual exposure, 2 = Auto bracket*/
    ES_U8 whiteBalance;       /* Indicates the white balance mode set when the image was shot.
                                            0 = Auto white balance ,1 = Manual white balance */
} ISP_DCF_UPDATE_INFO_S;

typedef struct esISP_DCF_INFO_S {
    ISP_DCF_CONST_INFO_S ispDCFConstInfo;
    ISP_DCF_UPDATE_INFO_S ispDCFUpdateInfo;
} ISP_DCF_INFO_S;

typedef struct esJPEG_DCF_S {
    ES_U8 captureTime[DCF_CAPTURE_TIME_LENGTH]; /*The date and time when the picture data was generated*/
    ES_BOOL bFlash;                             /*whether the picture is captured when a flash lamp is on*/
    ES_U32 digitalZoomRatio;                    /*Indicates the digital zoom ratio when the image was shot.
                                                              If the numerator of the recorded value is 0, this indicates that digital zoom
                                                       was not used.*/
    ISP_DCF_INFO_S ispDCFInfo;

} JPEG_DCF_S;

typedef struct esISP_FRAME_INFO_S {
    ES_U32 ISO;          /* ISP internal ISO : Again*Dgain*ISPgain */
    ES_U32 exposureTime; /* Exposure time (reciprocal of shutter speed),unit is us */
    ES_U32 ispDgain;
    ES_U32 again;
    ES_U32 dgain;
    ES_U32 ratio[3];
    ES_U32 ispNrStrength;
    ES_U32 fNumber;  /* The actual F-number (F-stop) of lens when the image was taken */
    ES_U32 sensorID; /* which sensor is used */
    ES_U32 sensorMode;
    ES_U32 hmaxTimes; /* Sensor HmaxTimes,unit is ns */
    ES_U32 vcNum;     /* when dump wdr frame, which is long or short  exposure frame. */
} ISP_FRAME_INFO_S;

#define CFACOLORPLANE (3)
#define DNG_NP_SIZE (6)

typedef struct esDNG_RATIONAL_S {
    ES_U32 numerator;   /*represents the numerator of a fraction,*/
    ES_U32 denominator; /* the denominator. */
} DNG_RATIONAL_S;

typedef enum esFRAME_FLAG_E {
    FRAME_FLAG_SNAP_FLASH = 0x1 << 0,
    FRAME_FLAG_SNAP_CUR = 0x1 << 1,
    FRAME_FLAG_SNAP_REF = 0x1 << 2,
    FRAME_FLAG_SNAP_END = 0x1 << 31,
    FRAME_FLAG_BUTT
} FRAME_FLAG_E;

/* RGGB=4 */
#define ISP_WB_GAIN_NUM 4
/* 3*3=9 matrix */
#define ISP_CAP_CCM_NUM 9
typedef struct esISP_CONFIG_INFO_S {
    ES_U32 ISO;
    ES_U32 ispDgain;
    ES_U32 exposureTime;
    ES_U32 whiteBalanceGain[ISP_WB_GAIN_NUM];
    ES_U32 colorTemperature;
    ES_U16 capCCM[ISP_CAP_CCM_NUM];
} ISP_CONFIG_INFO_S;

typedef struct esVIDEO_SUPPLEMENT_S {
    ES_U64 haeIOVA;
} VIDEO_SUPPLEMENT_S;

typedef enum esCOLOR_GAMUT_E {
    COLOR_GAMUT_BT601 = 0,
    COLOR_GAMUT_BT709 = 1,
    COLOR_GAMUT_BT2020 = 2,
    COLOR_GAMUT_BT601_L = 3,
    COLOR_GAMUT_BT709_L = 4,
    COLOR_GAMUT_BT2020_L = 5,
    COLOR_GAMUT_USER = 6,
    COLOR_GAMUT_BUTT
} COLOR_GAMUT_E;

typedef struct esISP_COLORGAMMUT_INFO_S {
    COLOR_GAMUT_E colorGamut;
} ISP_COLORGAMMUT_INFO_S;

typedef enum esDYNAMIC_RANGE_E {
    DYNAMIC_RANGE_NONE = 0,
    DYNAMIC_RANGE_HDR10 = 1,
    DYNAMIC_RANGE_BUTT,
} DYNAMIC_RANGE_E;

typedef struct esVIDEO_FRAME_S {
    ES_U32 width;
    ES_U32 height;
    VIDEO_FIELD_E field;
    PIXEL_FORMAT_E pixelFormat;
    DYNAMIC_RANGE_E dynamicRange;
    COLOR_GAMUT_E colorGamut;
    ES_U32 stride[3];
    ES_U32 offset[3];
    ES_U64 fd;
    ES_U32 maxLuminance;
    ES_U32 minLuminance;
    ES_U64 PTS;
    ES_U64 privateData;
    VIDEO_SUPPLEMENT_S supplement;
} VIDEO_FRAME_S;

typedef struct esVIDEO_FRAME_INFO_S {
    VIDEO_FRAME_S videoFrame;
    VB_POOL poolId;
    MOD_ID_E modId;
} VIDEO_FRAME_INFO_S;

typedef struct esVB_CAL_CONFIG_S {
    ES_U32 vbSize;
    ES_U32 headStride;
    ES_U32 headSize;
    ES_U32 headYSize;
    ES_U32 mainStride;
    ES_U32 mainSize;
    ES_U32 mainYSize;
    ES_U32 extStride;
    ES_U32 extYSize;
} VB_CAL_CONFIG_S;

typedef struct esLDC_ATTR_S {
    ES_BOOL bAspect;        /* RW;Range: [0, 1];Whether aspect ration  is keep */
    ES_S32 xRatio;          /* RW; Range: [0, 100]; field angle ration of  horizontal,valid when bAspect=0.*/
    ES_S32 yRatio;          /* RW; Range: [0, 100]; field angle ration of  vertical,valid when bAspect=0.*/
    ES_S32 xYRatio;         /* RW; Range: [0, 100]; field angle ration of  all,valid when bAspect=1.*/
    ES_S32 centerXOffset;   /* RW; Range: [-511, 511]; horizontal offset of the image distortion center relative to
                                   image center.*/
    ES_S32 centerYOffset;   /* RW; Range: [-511, 511]; vertical offset of the image distortion center relative to
                                   image center.*/
    ES_S32 distortionRatio; /* RW; Range: [-300, 500]; LDC Distortion ratio.When spread on,distortionRatio range
                                   should be [0, 500]*/
} LDC_ATTR_S;

typedef struct esLDCV2_ATTR_S {
    ES_S32 focalLenX;  /*RW; focal length in horizontal direction, with 2 decimal numbers*/
    ES_S32 focalLenY;  /*RW; focal length in vertical direction, with 2 decimal numbers*/
    ES_S32 coorShiftX; /*RW; coordinate of image center, with 2 decimal numbers*/
    ES_S32 coorShiftY; /*RW; Y coordinate of image center, with 2 decimal numbers*/
    ES_S32 srcCaliRatio[SRC_LENS_COEF_SEG][SRC_LENS_COEF_NUM]; /*RW; lens distortion coefficients of the source
                                                                       image, with 5 decimal numbers*/
    ES_S32 srcJunPt;                                           /*RW; Junction Point of the two segments*/
    ES_S32 dstCaliRatio[DST_LENS_COEF_SEG]
                       [DST_LENS_COEF_NUM]; /*RW; lens distortion coefficients, with 5 decimal numbers*/
    ES_S32 dstJunPt[DST_LENS_COEF_SEG - 1]; /*RW; Junction Point of the three segments*/
} LDCV2_ATTR_S;

typedef enum esLDC_VIEW_TYPE_E {
    LDC_VIEW_TYPE_ALL =
        0, /* View scale all but x and y independtly, this will keep both x and y axis ,but corner maybe lost*/
    LDC_VIEW_TYPE_CROP = 1, /* Not use view scale, this will lost some side and corner */

    LDC_VIEW_TYPE_BUTT,
} LDC_VIEW_TYPE_E;

typedef struct esLDCV3_ATTR_S {
    LDC_VIEW_TYPE_E viewType;
    ES_S32 centerXOffset;
    ES_S32 centerYOffset;
    ES_S32 distortionRatio;
    ES_S32 minRatio;
} LDCV3_ATTR_S;

typedef enum esROTATION_VIEW_TYPE_E {
    ROTATION_VIEW_TYPE_ALL = 0,     /* View all source Image,no lose*/
    ROTATION_VIEW_TYPE_TYPICAL = 1, /* View from rotation Image with source size,same lose*/
    ROTATION_VIEW_TYPE_INSIDE = 2,  /* View with no black section,all  in dest Image*/
    ROTATION_VIEW_TYPE_BUTT,
} ROTATION_VIEW_TYPE_E;

typedef struct esROTATION_EX_S {
    ROTATION_VIEW_TYPE_E viewType; /*RW;Range: [0, 2];Rotation mode*/
    ES_U32 angle;                  /*RW;Range: [0,360];Rotation Angle:[0,360]*/
    ES_S32 centerXOffset; /*RW;Range: [-511, 511];Horizontal offset of the image distortion center relative to image
                                 center.*/
    ES_S32 centerYOffset; /*RW;Range: [-511, 511];Vertical offset of the image distortion center relative to image
                                 center.*/
    SIZE_S destSize;      /*RW;Dest size of any angle rotation*/
} ROTATION_EX_S;

typedef enum esWDR_MODE_E {
    WDR_MODE_NONE = 0,
    WDR_MODE_BUILT_IN = 1,
    WDR_MODE_QUDRA = 2,
    WDR_MODE_2To1_LINE = 3,
    WDR_MODE_2To1_FRAME = 4,
    WDR_MODE_2To1_FRAME_FULL_RATE = 5,
    WDR_MODE_3To1_LINE = 6,
    WDR_MODE_3To1_FRAME = 7,
    WDR_MODE_3To1_FRAME_FULL_RATE = 8,
    WDR_MODE_4To1_LINE = 9,
    WDR_MODE_4To1_FRAME = 10,
    WDR_MODE_4To1_FRAME_FULL_RATE = 11,
    WDR_MODE_BUTT,
} WDR_MODE_E;

typedef enum esFRAME_INTERRUPT_TYPE_E {
    FRAME_INTERRUPT_START = 0,
    FRAME_INTERRUPT_EARLY = 1,
    FRAME_INTERRUPT_EARLY_END = 2,
    FRAME_INTERRUPT_BUTT,
} FRAME_INTERRUPT_TYPE_E;

typedef struct esFRAME_INTERRUPT_ATTR_S {
    FRAME_INTERRUPT_TYPE_E intType;
    ES_U32 earlyLine;
} FRAME_INTERRUPT_ATTR_S;

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* _ES_COMM_VIDEO_H_ */
